package com.mortals.xhx.busiz.web;

import cn.hutool.core.codec.Base64;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.web.BaseJsonBodyController;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.model.vo.MatterInfo;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.site.model.SiteMatterEntity;
import com.mortals.xhx.module.site.model.SiteMatterQuery;
import com.mortals.xhx.module.site.service.SiteMatterService;
import com.mortals.xhx.module.window.model.*;
import com.mortals.xhx.module.window.service.WindowBusinessService;
import com.mortals.xhx.module.window.service.WindowMatterService;
import com.mortals.xhx.module.window.service.WindowService;
import lombok.extern.apachecommons.CommonsLog;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpResponse;
import org.apache.http.client.CookieStore;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.mortals.framework.ap.SysConstains.MESSAGE_INFO;
import static com.mortals.framework.ap.SysConstains.PAGEINFO_KEY;

/**
 * 测试接口
 *
 * @author:
 * @date: 2022/12/14 9:18
 */
@RestController
@RequestMapping("test")
@Slf4j
public class DemoWebApiController {

    @Autowired
    private WindowService windowService;
    @Autowired
    private SiteMatterService siteMatterService;
    @Autowired
    private WindowMatterService windowMatterService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private MatterService matterService;


    @PostMapping(value = "testGov")
    @UnAuth
    public Rest<String> testGov(@RequestBody MatterQuery query) {
        String resp = HttpUtil.get(query.getUrl());
        return Rest.ok(resp);
    }

    @PostMapping(value = "testCookie")
    @UnAuth
    public Rest<String> testCookie(@RequestBody MatterQuery query) {
        log.info("测试cookie");
        HttpClient http = null;
        CookieStore httpCookieStore = new BasicCookieStore();
        http = HttpClientBuilder.create().setDefaultCookieStore(httpCookieStore).build();

        /* do stuff */
        HttpGet httpRequest = new HttpGet(query.getUrl());
        HttpResponse httpResponse = null;
        try {
            httpResponse = http.execute(httpRequest);
            byte[] data = EntityUtils.toByteArray(httpResponse.getEntity());
            String encode = Base64.encode(data);

            //String content = EntityUtils.toString(entity, charset);
            //httpResponse.getEntity()
            //httpResponse.toString()
        } catch (Exception e) {
            log.error("异常", e);
        }
        /* check cookies */
        String cookieStr = Arrays.asList(httpCookieStore.getCookies()).stream().map(item -> JSON.toJSONString(item)).collect(Collectors.joining("|"));
        log.info("cookies:{}", cookieStr);
        return Rest.ok(cookieStr);
    }

    @PostMapping(value = "reEventShow")
    @UnAuth
    public Rest<String> reEventShow() {
        List<SiteMatterEntity> siteMatterEntities = siteMatterService.find(new SiteMatterQuery());
        for (SiteMatterEntity siteMatterEntity : siteMatterEntities) {
            if(ObjectUtils.isEmpty(siteMatterEntity.getEventTypeShow())){
                MatterEntity matterEntity = matterService.get(siteMatterEntity.getMatterId());
                if(!ObjectUtils.isEmpty(matterEntity)&&!ObjectUtils.isEmpty(matterEntity.getEventTypeShow())){
                    siteMatterEntity.setEventTypeShow(matterEntity.getEventTypeShow());
                    siteMatterService.update(siteMatterEntity);
                }
            }
        }

        return Rest.ok();
    }


    @PostMapping(value = "reDepts")
    @UnAuth
    public Rest<String> reDepts() {
        log.info("更新部门与设备关联");
        //更新窗口部门
        List<WindowEntity> windowEntities = windowService.find(new WindowQuery());
        for (WindowEntity windowEntity : windowEntities) {
            String deptName = windowEntity.getDeptName();
            //根据部门名称查询部门
            DeptEntity deptEntity = deptService.selectOne(new DeptQuery().name(deptName+"%"));
            if (!ObjectUtils.isEmpty(deptEntity)) {
                if (deptEntity.getId() != windowEntity.getDeptId()&&deptEntity.getName().trim().equals(windowEntity.getDeptName().trim())) {
                    log.info("部门:{},更新部门id:orgin deptId:{} ,updateDeptId:{}", deptName, windowEntity.getDeptId(), deptEntity.getId());
                    WindowEntity temp = new WindowEntity();
                    temp.setId(windowEntity.getId());
                    temp.setDeptId(deptEntity.getId());
                    windowService.getDao().update(temp);

                    // windowService.update(windowEntity);
                }
            }
        }
        //更新窗口事项中的部门id
        List<WindowMatterEntity> windowMatterEntities = windowMatterService.find(new WindowMatterQuery());
        for (WindowMatterEntity windowMatterEntity : windowMatterEntities) {
            DeptEntity deptEntity = deptService.selectOne(new DeptQuery().name(windowMatterEntity.getDeptName()+"%"));
            if (!ObjectUtils.isEmpty(deptEntity)) {
                if (deptEntity.getId() != windowMatterEntity.getDeptId()&&windowMatterEntity.getDeptName().trim().equals(deptEntity.getName())) {
                    WindowMatterEntity temp = new WindowMatterEntity();
                    temp.setId(windowMatterEntity.getId());
                    temp.setDeptId(deptEntity.getId());
                    temp.setDeptCode(deptEntity.getDeptNumber());
                    windowMatterService.getDao().update(temp);
                    // windowMatterService.update(windowMatterEntity);
                }
            }
        }

        List<SiteMatterEntity> siteMatterEntities = siteMatterService.find(new SiteMatterQuery());
        for (SiteMatterEntity siteMatterEntity : siteMatterEntities) {
            DeptEntity deptEntity = deptService.selectOne(new DeptQuery().name(siteMatterEntity.getDeptName()+"%"));
            if (!ObjectUtils.isEmpty(deptEntity)) {
                if (deptEntity.getId() != siteMatterEntity.getDeptId()&&siteMatterEntity.getDeptName().trim().equals(deptEntity.getName().trim())) {
                    SiteMatterEntity temp = new SiteMatterEntity();
                    temp.setId(siteMatterEntity.getId());
                    temp.setDeptId(deptEntity.getId());
                    temp.setDeptCode(deptEntity.getDeptNumber());
                    siteMatterService.getDao().update(temp);

                    //siteMatterService.update(siteMatterEntity);
                }
            }
        }

        return Rest.ok();
    }


    public static void main(String[] args) {
        HttpClient http = null;
        CookieStore httpCookieStore = new BasicCookieStore();
        http = HttpClientBuilder.create().setDefaultCookieStore(httpCookieStore).build();

        /* do stuff */
        HttpGet httpRequest = new HttpGet("http://zxbl.sczwfw.gov.cn/app/account/imageCode");
        HttpResponse httpResponse = null;
        try {
            httpResponse = http.execute(httpRequest);
            byte[] data = EntityUtils.toByteArray(httpResponse.getEntity());
            String encode = Base64.encode(data);

            log.info("encode64:{}", encode);
            // String resp = com.mortals.framework.util.HttpUtil.processMultipartResponse(httpResponse);
            // log.info("resp:{}", resp);
            //String content = EntityUtils.toString(entity, charset);
            //httpResponse.getEntity()
            //httpResponse.toString()
        } catch (Exception e) {
            log.error("异常", e);
        }
        /* check cookies */
        String cookieStr = Arrays.asList(httpCookieStore.getCookies()).stream().map(item -> JSON.toJSONString(item)).collect(Collectors.joining("|"));
        // log.info("cookies:{}", cookieStr);
    }


}
