package com.mortals.xhx.daemon.applicationservice;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.springcloud.service.IApplicationStartedService;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.common.utils.CustomerWebSocketClient;
import com.mortals.xhx.common.utils.ImageToBase64Converter;
import com.mortals.xhx.module.apply.model.MatterApplyDatumEntity;
import com.mortals.xhx.module.apply.model.MatterApplyDatumQuery;
import com.mortals.xhx.module.apply.model.MatterApplyEntity;
import com.mortals.xhx.module.apply.service.MatterApplyDatumService;
import com.mortals.xhx.module.apply.service.MatterApplyService;
import com.mortals.xhx.module.matter.model.MatterDatumEntity;
import com.mortals.xhx.module.matter.model.MatterDatumQuery;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.net.URI;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.mortals.xhx.common.key.RedisKey.SINGLE_MATTER_APPLY;

/**
 * 单事项材料预审
 */
@Component
@Slf4j
public class SingleApplyreviewService implements IApplicationStartedService {
    protected Boolean stopped = false;

    @Autowired
    private ICacheService cacheService;
    @Autowired
    private MatterApplyDatumService matterApplyDatumService;
    @Autowired
    private MatterDatumService matterDatumService;
    @Autowired
    private MatterApplyService matterApplyService;

    @Value("${upload.path}")
    @Getter
    private String filePath;
    @Value("${webSocket.url}")
    @Getter
    private String apiUrl;


    @Override
    public void start() {
        log.info("开始服务..[单事项材料预审]");

        Thread sendThread = new Thread(new Runnable() {
            @Override
            public void run() {
                int waitTime = 1000;
                while (!stopped) {
                    try {
                        URI uri = new URI(apiUrl);

                        Long applyId = cacheService.brpop(SINGLE_MATTER_APPLY, 60, Long.class);
                        List<MatterApplyDatumEntity> datumList = matterApplyDatumService.find(new MatterApplyDatumQuery().applyId(applyId));
                        if(CollectionUtils.isNotEmpty(datumList)){
                            String rootPath = filePath.endsWith("/") ? filePath : filePath + "/";

                            List<MatterDatumEntity> datumEntityList = matterDatumService.find(new MatterDatumQuery().matterId(datumList.get(0).getMatterId()));
                            Map<Long,MatterDatumEntity> datumEntityMap = datumEntityList.stream().collect(Collectors.toMap(datumEntity->datumEntity.getId(), Function.identity()));
                            for(MatterApplyDatumEntity item:datumList){
                                CustomerWebSocketClient webSocketClient = new CustomerWebSocketClient(uri);
                                webSocketClient.connect();
                                webSocketClient.setMatterApplyDatumService(matterApplyDatumService);
                                webSocketClient.setMatterApplyService(matterApplyService);
                                if(!webSocketClient.isOpen()){
                                    Thread.sleep(2000);
                                }

                                webSocketClient.setApplyDatumId(item.getId());
                                //TODO
                                MatterDatumEntity matterDatumEntity = datumEntityMap.get(item.getDatumId());
                                Map<String, String> processUserImage = new HashMap<>();
                                processUserImage.put("action", "pre_review");
                                processUserImage.put("system_prompt", matterDatumEntity.getSystemPrompt());
                                processUserImage.put("user_prompt", matterDatumEntity.getPrompt());
                                String template_image_url = ImageToBase64Converter.convertToBase64(rootPath + matterDatumEntity.getMatterDatumFileList().get(0).getPreviewUrl());
                                processUserImage.put("template_image_url", template_image_url);
                                String user_image_url = ImageToBase64Converter.convertToBase64(rootPath + item.getFileUrl());
                                processUserImage.put("user_image_url", user_image_url);
                                processUserImage.put("serial_number", item.getId().toString());
                                if(webSocketClient.isOpen()) {
                                    webSocketClient.setCurrentTime(System.currentTimeMillis());
                                    webSocketClient.send(JSONObject.toJSONString(processUserImage));
                                    //log.info("发送消息: " + JSONObject.toJSONString(processUserImage));
                                }
                                try {
                                    Thread.sleep(15000);
                                } catch (InterruptedException e2) {
                                }
                                webSocketClient.close();
                            }
                        }

                    }catch (Exception e){
                        try {
                            Thread.sleep(waitTime);
                        } catch (InterruptedException e2) {
                        }
                    }
                }
            }
        });

        sendThread.start();
    }

    @Override
    public void stop() {
        log.info("停止服务..");
        this.stopped = true;
    }

    @Override
    public int getOrder() {
        return 10;
    }
}
