package com.mortals.xhx.module.site.service.impl;

import cn.hutool.core.util.URLUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.impl.AbstractCRUDCacheServiceImpl;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.utils.MatterHtmlParseUtil;
import com.mortals.xhx.module.area.model.AreaEntity;
import com.mortals.xhx.module.area.model.AreaQuery;
import com.mortals.xhx.module.area.service.AreaService;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.model.model.ModelEntity;
import com.mortals.xhx.module.model.model.ModelQuery;
import com.mortals.xhx.module.model.service.ModelService;
import com.mortals.xhx.module.site.dao.SiteDao;
import com.mortals.xhx.module.site.model.*;
import com.mortals.xhx.module.site.model.vo.SiteAreaVo;
import com.mortals.xhx.module.site.service.SiteBusinessService;
import com.mortals.xhx.module.site.service.SiteMatterService;
import com.mortals.xhx.module.site.service.SiteService;
import com.mortals.xhx.module.site.service.SiteThemeMatterService;
import com.mortals.xhx.module.window.model.WindowEntity;
import com.mortals.xhx.module.window.service.WindowService;
import com.mortals.xhx.module.workman.model.WorkmanEntity;
import com.mortals.xhx.module.workman.service.WorkmanService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

import static com.mortals.xhx.common.key.Constant.PARAM_SERVER_HTTP_URL;
import static com.mortals.xhx.common.key.Constant.USER_SITE_TREE;

/**
 * SiteService
 * 站点 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("siteService")
@Slf4j
public class SiteServiceImpl extends AbstractCRUDCacheServiceImpl<SiteDao, SiteEntity, Long> implements SiteService {
    // private List<SiteTreeSelect> siteTreeList;
    /**
     * 根据用户id 暂存对应站点树 默认0为全站点树
     */
    private Map<Long, List<SiteTreeSelect>> siteTreeMap = new HashMap<>();

    @Autowired
    private AreaService areaService;
    @Autowired
    private ModelService modelService;
//    @Autowired
//    private IUserFeign userFeign;
    @Autowired
    private UserService userService;
    @Autowired
    private DeptService deptService;

    @Autowired
    private MatterService matterService;
    @Autowired
    private WindowService windowService;
    @Autowired
    private SiteBusinessService siteBusinessService;
    @Autowired
    private SiteThemeMatterService siteThemeMatterService;
    @Autowired
    private SiteMatterService siteMatterService;
    @Autowired
    private WorkmanService workmanService;
//    @Autowired
//    private IDeviceFeign deviceFeign;

    private volatile Boolean refresh = false;


    @Override
    protected void updateBefore(SiteEntity entity, Context context) throws AppException {
        //校验下前后名称是否一直 如果不一致 同步修改所以关联
        SiteEntity oldSite = this.get(entity.getId(), context);
        if (!oldSite.getSiteName().equals(entity.getSiteName())) {
            //todo
            //更新关联站点名称
            updateAssociateSiteName(oldSite, entity, context);
            refresh = true;
        }

        if (!oldSite.getEnglishName().equals(entity.getEnglishName())) {
            refresh = true;
        }

        super.updateBefore(entity, context);
    }


    private void updateAssociateSiteName(SiteEntity oldSite, SiteEntity newSite, Context context) {
        //更新窗口站点名称
        WindowEntity windowEntity = new WindowEntity();
        windowEntity.setSiteName(newSite.getSiteName());
        WindowEntity condition = new WindowEntity();
        condition.setSiteName(oldSite.getSiteName());
        windowService.updateBatch(windowEntity, condition, context);

        //更新站点业务名称
        SiteBusinessEntity siteBusinessEntity = new SiteBusinessEntity();
        siteBusinessEntity.setSiteName(newSite.getSiteName());
        SiteBusinessEntity siteBusinessCondition = new SiteBusinessEntity();
        siteBusinessCondition.setSiteName(oldSite.getSiteName());
        siteBusinessService.updateBatch(siteBusinessEntity, siteBusinessCondition, context);
        //更新站点事项名称
        SiteMatterEntity siteMatterEntity = new SiteMatterEntity();
        siteMatterEntity.setSiteName(newSite.getSiteName());
        SiteMatterEntity siteMatterCondition = new SiteMatterEntity();
        siteMatterCondition.setSiteName(oldSite.getSiteName());
        siteMatterService.updateBatch(siteMatterEntity, siteMatterCondition, context);

        //更新站点主题事项名称
        SiteThemeMatterEntity siteThemeMatterEntity = new SiteThemeMatterEntity();
        siteThemeMatterEntity.setSiteName(newSite.getSiteName());
        SiteThemeMatterEntity siteThemeMatterCondition = new SiteThemeMatterEntity();
        siteThemeMatterCondition.setSiteName(oldSite.getSiteName());
        siteThemeMatterService.updateBatch(siteThemeMatterEntity, siteThemeMatterCondition, context);

        //更新工作人员站点

        WorkmanEntity workmanEntity = new WorkmanEntity();
        workmanEntity.setSiteName(newSite.getSiteName());
        WorkmanEntity workmanEntityCondition = new WorkmanEntity();
        workmanEntityCondition.setSiteName(oldSite.getSiteName());
        workmanService.updateBatch(workmanEntity, workmanEntityCondition, context);

    }

    @Override
    protected void validData(SiteEntity entity, Context context) throws AppException {
        if (ObjectUtils.isEmpty(entity.getAreaCode())) {
            throw new AppException("区域编码不能为空！");
        }
        AreaEntity areaEntity = areaService.getCache(entity.getAreaCode());
        if (ObjectUtils.isEmpty(areaEntity)) {
            throw new AppException(String.format("区域不存在！区域编码：%s", entity.getAreaCode()));
        }
        super.validData(entity, context);
    }

    @Override
    protected void saveBefore(SiteEntity entity, Context context) throws AppException {
        List<SiteEntity> siteEntities = this.find(new SiteQuery().areaCode(entity.getAreaCode()));
        entity.setSiteCode(entity.getAreaCode() + Constant.SITE_SPLIT_MARK + StringUtils.lpad(siteEntities.size() + 1, Constant.SITE_CODE_NUM));
        super.saveBefore(entity, context);
    }

    @Override
    protected void saveAfter(SiteEntity entity, Context context) throws AppException {
        //刷新站点树
        //ThreadPool.getInstance().execute(new SyncTreeSiteThread(this,context));
//        Rest<String> rest = userFeign.synchSiteAuth();
//        if (rest.getCode().equals(YesNoEnum.YES.getValue())) {
//            UserPdu userPdu = new UserPdu();
//            userPdu.setPage(1);
//            userPdu.setSize(-1);
//            Rest<RespData<List<UserPdu>>> list = userFeign.list(userPdu);
//            userService.updateUserList(list.getData().getData());
//        }
        //更新同步部门相关
        deptService.syncDeptBySiteId(entity, context);
        super.saveAfter(entity, context);
    }


    @Override
    public List<SiteTreeSelect> siteTree(Context context) {
        Map<String, AreaEntity> areaMap = new HashMap<>();
        SiteQuery siteQuery = new SiteQuery();
        if (context.getUser().getSiteIds() != null) {
            Set<String> siteSet = Arrays.stream(context.getUser().getSiteIds().split(",")).filter(f -> !f.equals("")).collect(Collectors.toSet());
            if (!ObjectUtils.isEmpty(siteSet)) {
                List<Long> siteIdList = siteSet.stream().map(Long::parseLong).collect(Collectors.toList());
                if (!ObjectUtils.isEmpty(siteIdList)) {
                    siteQuery.idList(siteIdList);
                }
            }
        }

        //查詢指定的站點ids
        log.info(String.format("siteQuery==>%s", JSON.toJSONString(siteQuery)));
        List<SiteEntity> siteList = this.find(siteQuery);
        //如果是管理员 默认全部站点
        if (context.getUser().isAdmin()) {
            log.info("user is admin !");
            siteList = this.find(new SiteQuery());
        }

        //turn to sitemap
        Map<String, SiteEntity> siteMap = siteList.parallelStream().collect(Collectors.toMap(x -> x.getSiteCode(), y -> y, (o, n) -> n));
        //遍历过滤站点树
        for (SiteEntity siteEntity : siteList) {
            AreaEntity areaEntity = areaService.getExtCache(siteEntity.getAreaCode());
            //根据区域添加父节点
            if (!ObjectUtils.isEmpty(areaEntity)
                    && !ObjectUtils.isEmpty(areaEntity.getPid())) {
                //递归查找父节点并添加
                areaMap.put(areaEntity.getAreaCode(), areaEntity);
                //递归
                recursionFn(areaMap, areaEntity);
            }
        }
        //log.info("rebuildAreaMap==>{}", JSON.toJSONString(areaMap));
        //areaMap 为当前站点相关区域树结构 已经剔除不必要的分支结构
        return buildSiteTreeSelect(areaMap, siteMap);
    }

    public List<SiteTreeSelect> buildSiteTreeSelect(Map<String, AreaEntity> areaMap, Map<String, SiteEntity> siteMap) {
        List<AreaEntity> areaList = areaMap.values().stream().collect(Collectors.toList());
        List<AreaEntity> returnList = new ArrayList<>();
        //区域id
        List<String> tempList = areaList.stream().map(AreaEntity::getIid).collect(Collectors.toList());
        for (AreaEntity areaEntity : areaList) {
            //查询当前区域列表中最顶级的节点 并构建树
            if (!tempList.contains(areaEntity.getPid())) {
                recursion(areaList, areaEntity);
                returnList.add(areaEntity);
            }
        }
        //区域树
        if (returnList.isEmpty()) {
            returnList = areaList;
        }

        //构建站点树
        List<SiteTreeSelect> collect = returnList.parallelStream().map(item ->
                new SiteTreeSelect(item, siteMap)
        ).collect(Collectors.toList());
        //this.siteTreeList = collect;
        return collect;

    }

    private void recursion(List<AreaEntity> list, AreaEntity entity) {
        // 得到子节点列表
        List<AreaEntity> childList = getChildList(list, entity);
        entity.setChildren(childList);
        for (AreaEntity tChild : childList) {
            if (hasChild(list, tChild)) {
                recursion(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<AreaEntity> getChildList(List<AreaEntity> list, AreaEntity entity) {
        return list.stream()
                .filter(item -> item.getPid().equals(entity.getIid()))
                .collect(Collectors.toList());
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<AreaEntity> list, AreaEntity t) {
        return getChildList(list, t).size() > 0 ? true : false;
    }


    /**
     * 递归列表
     */
    private void recursionFn(Map<String, AreaEntity> areaMap, AreaEntity t) {
        // 得到父节点并加入
        AreaEntity areaEntity = areaService.selectOne(new AreaQuery().iid(t.getPid()));
        if (!ObjectUtils.isEmpty(areaEntity)) {
            areaMap.put(areaEntity.getAreaCode(), areaEntity);
            recursionFn(areaMap, areaEntity);
        }
    }


    @Override
    protected void updateAfter(SiteEntity entity, Context context) throws AppException {
        super.updateAfter(entity, context);
        //刷新站点树
//        Rest<String> rest = userFeign.synchSiteAuth();
//        if (rest.getCode().equals(YesNoEnum.YES.getValue())) {
//            UserPdu userPdu = new UserPdu();
//            userPdu.setPage(1);
//            userPdu.setSize(-1);
//            Rest<RespData<List<UserPdu>>> list = userFeign.list(userPdu);
//            userService.updateUserList(list.getData().getData());
//        }
//        if (refresh) {
//            //通知设备更新站点信息
//            DevicePdu devicePdu = new DevicePdu();
//            devicePdu.setSiteId(entity.getId());
//            Rest<Void> rest1 = deviceFeign.refreshMessage(devicePdu);
//            log.info("调用刷新结果：{}", JSON.toJSONString(rest1));
//            refresh = false;
//        }

    }


    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {

//        Rest<String> rest = userFeign.synchSiteAuth();
//        if (rest.getCode().equals(YesNoEnum.YES.getValue())) {
//            UserPdu userPdu = new UserPdu();
//            userPdu.setPage(1);
//            userPdu.setSize(-1);
//            Rest<RespData<List<UserPdu>>> list = userFeign.list(userPdu);
//
//            userService.updateUserList(list.getData().getData());
//        }
        super.removeAfter(ids, context, result);
    }

    @Override
    public List<SiteTreeSelect> getSiteTree(Context context) {
        List<SiteTreeSelect> siteTreeSelects = siteTreeMap.get(context.getUser().getId());
        List<SiteTreeSelect> collect = getSiteTreeSelects(context.getUser().getId().toString());
        log.info("siteTree:{}", JSON.toJSONString(collect));
        if (!ObjectUtils.isEmpty(collect)) {
            return collect;
        } else {
            //反全站点树
            return getSiteTreeSelects("0");
        }

        //如果所属站点为空返回全节点树
//        if (!ObjectUtils.isEmpty(siteTreeSelects)) {
//            return siteTreeSelects;
//        } else {
//            return siteTreeMap.get(1);
//        }
    }

    private List<SiteTreeSelect> getSiteTreeSelects(String userId) {
        String siteTreeSelectStr = cacheService.hget(USER_SITE_TREE, userId, String.class);
        log.info("userId:{},siteTreeSelectStr:{}", userId, siteTreeSelectStr);
        //反序列化树对象
        if (ObjectUtils.isEmpty(siteTreeSelectStr)) {
            return new ArrayList<>();
        }

        JSONArray jsonArray = JSON.parseArray(siteTreeSelectStr);
        if (ObjectUtils.isEmpty(jsonArray)) {
            return new ArrayList<>();
        }
        List<SiteTreeSelect> collect = jsonArray.stream().map(item -> {
            SiteTreeSelect siteTreeSelect = JSON.parseObject(item.toString(), SiteTreeSelect.class);
            return siteTreeSelect;
        }).collect(Collectors.toList());
        return collect;
    }

    @Override
    public List<SiteEntity> getFlatSitesByAreaCode(String areaCode, Context context) {
        if (ObjectUtils.isEmpty(areaCode)) {
            throw new AppException("区域编码不能为空！");
        }
        AreaEntity areaEntity = areaService.getExtCache(areaCode);
        if (ObjectUtils.isEmpty(areaEntity)) {
            throw new AppException(String.format("区域不存在！区域编码：%s", areaCode));
        }
        String matchCode = areaCode.replaceAll("(0)+$", "");
        SiteQuery siteQuery = new SiteQuery();
        siteQuery.setAreaCode(matchCode + "%");
        List<SiteEntity> siteEntities = this.find(siteQuery);

        return siteEntities;
    }

    @Override
    public List<SiteEntity> getFlatSitesBySiteId(Long siteId, Context context) {
        List<SiteEntity> siteList = new ArrayList<>();
        if (ObjectUtils.isEmpty(siteId)) {
            throw new AppException("站点Id不能为空！");
        }
        SiteEntity siteCache = this.getCache(siteId.toString());
        if (ObjectUtils.isEmpty(siteCache)) {
            throw new AppException("站点不存在，siteId:" + siteId);
        }
        //siteList.add(siteCache);
        AreaEntity areaEntity = areaService.getExtCache(siteCache.getAreaCode());
        if (ObjectUtils.isEmpty(areaEntity)) {
            throw new AppException(String.format("区域不存在！区域编码：%s", siteCache.getAreaCode()));
        }

        String matchCode = siteCache.getAreaCode().replaceAll("(0)+$", "");
        SiteQuery siteQuery = new SiteQuery();
        siteQuery.setAreaCode(matchCode + "%");
        List<SiteEntity> siteEntities = this.find(siteQuery);

        if (!ObjectUtils.isEmpty(siteEntities)) {
            siteList.addAll(siteEntities);
        }
        return siteList;
    }

    @Override
    public Rest<List<SiteAreaVo>> getAreaSitesBySite(SiteQuery query, Context context) {
        List<SiteAreaVo> list = new ArrayList<>();
        //获取当前节点的第一层子节点
        if (ObjectUtils.isEmpty(query.getAreaName())) {
            throw new AppException("区域名称查询参数不能为空！");
        }
        //SiteEntity siteCache = this.getCache(query.getId().toString());
        AreaEntity areaEntity = areaService.selectOne(new AreaQuery().name(query.getAreaName()), context);
        if (ObjectUtils.isEmpty(areaEntity)) {
            throw new AppException("区域名称不存在！");
        }
        if ("False".equalsIgnoreCase(areaEntity.getHaveSonArea())) {
            throw new AppException("当前站点无子区域！");
        }
        //两层结构{xxx:[{sitename:xxx},{}...],bbb:[{sitename:xxx},{}...]}
        AreaQuery areaQuery = new AreaQuery();
        areaQuery.setPid(areaEntity.getIid());
        List<AreaEntity> childList = areaService.find(areaQuery, context);
        for (AreaEntity child : childList) {
            SiteAreaVo siteAreaVo = new SiteAreaVo();

            String matchCode = child.getAreaCode().replaceAll("(0)+$", "");
            SiteQuery siteQuery = new SiteQuery();
            siteQuery.setAreaCode(matchCode + "%");
            if (!ObjectUtils.isEmpty(query.getSiteName())) {
                siteQuery.setSiteName(query.getSiteName());
            }
            List<SiteEntity> siteEntities = this.find(siteQuery);
            if (!ObjectUtils.isEmpty(siteEntities)) {
                siteAreaVo.setAreaCode(child.getAreaCode());
                siteAreaVo.setAreaName(child.getName());
                siteAreaVo.setSiteList(siteEntities);
                list.add(siteAreaVo);
                //map.put(child.getName(), siteEntities);
            }
        }
        return Rest.ok(list);
    }

    private boolean getSite(String rootAreaCode, String areaCode, List<SiteEntity> siteList, SiteEntity siteEntity) {
        // AreaEntity areaTemp = areaService.getExtCache(siteEntity.getAreaCode());
        if (rootAreaCode.equals(areaCode)) {
            siteList.add(siteEntity);
            //log.info("add site:{}", siteEntity.getSiteCode());
            return true;
        } else {
            AreaEntity areaTemp = areaService.getExtCache(areaCode);
            if (!ObjectUtils.isEmpty(areaTemp) && !ObjectUtils.isEmpty(areaTemp.getPid())) {
                //根据子节点查找父节点
                AreaEntity areaEntity = areaService.selectOne(new AreaQuery().iid(areaTemp.getPid()));
                if (!ObjectUtils.isEmpty(areaEntity)) {
                    return getSite(rootAreaCode, areaEntity.getAreaCode(), siteList, siteEntity);
                }
            }
            return false;
        }
    }

    @Override
    public void setSiteTree(List<SiteTreeSelect> list, Context context) {
        siteTreeMap.put(context.getUser().getId(), list);
        //存放到redis中去
        cacheService.hsetnx(USER_SITE_TREE, context.getUser().getId().toString(), JSON.toJSONString(list));
    }

    @Override
    public void removeSiteTree(List<SiteTreeSelect> list, Context context) {
        siteTreeMap.remove(context.getUser().getId());
        cacheService.hdel(USER_SITE_TREE, context.getUser().getId().toString());

    }

    @Override
    public List<SiteEntity> getListAllInfoByQuery(SiteQuery query, Context context) throws AppException {
        List<SiteEntity> list = this.find(query, context);
        if (CollectionUtils.isNotEmpty(list)) {
            list.stream().forEach(item -> {
                ModelQuery modelQuery = new ModelQuery();
                List<String> modelIds = StringUtils.converStr2List(item.getModelIds());
                List<Long> ids = modelIds.stream().map(s -> DataUtil.converStr2Long(s.trim(), 0)).collect(Collectors.toList());
                if (ids.isEmpty()) {
                    ids.add(0l);
                }
                modelQuery.setIdList(ids);
                List<ModelEntity> modelEntityList = modelService.find(modelQuery);
                item.setModelList(modelEntityList);
            });
        }

        return list;
    }


    @Override
    protected void findAfter(SiteEntity params, PageInfo pageInfo, Context context, List<SiteEntity> list) throws AppException {
        list.stream().forEach(site -> changeUrlPath(site));
        super.findAfter(params, pageInfo, context, list);
    }

    @Override
    public void changeUrlPath(SiteEntity siteEntity) {
        if (!ObjectUtils.isEmpty(siteEntity) && !ObjectUtils.isEmpty(siteEntity.getLogoPath())) {
            String domain = GlobalSysInfo.getParamValue(PARAM_SERVER_HTTP_URL, "http://192.168.0.98:11091");
            String fullPath = URLUtil.completeUrl(domain, siteEntity.getLogoPath());
            siteEntity.setLogoFullPath(fullPath);
        }
    }


    @Override
    public Rest<List<MatterEntity>> getMatterAllListByGOV(Map<String, String> params, Integer pageNum, Context context) {
        //分页获取所有事项列表
        List<MatterEntity> allList = new ArrayList<>();
        String url = GlobalSysInfo.getParamValue(Constant.GOV_MATTER_PAGELIST_URL, "http://www.sczwfw.gov.cn/jiq/interface/item/tags");
        for (int i = 1; i <= pageNum; i++) {
            params.put("pageno", String.valueOf(i));
            Rest<List<MatterEntity>> rest = MatterHtmlParseUtil.getMatterList(params, url);
            if (rest.getCode() == YesNoEnum.YES.getValue() && !ObjectUtils.isEmpty(rest.getData())) {
                allList.addAll(rest.getData());
            }
        }
        return Rest.ok(allList);
    }

    private List<MatterEntity> getMatters(HashMap<String, String> params, Context context) {
        String url = GlobalSysInfo.getParamValue(Constant.GOV_MATTER_PAGELIST_URL, "http://www.sczwfw.gov.cn/jiq/interface/item/tags");
        Rest<Map<String, Integer>> restStat = MatterHtmlParseUtil.statSiteMatterCount(params, url);
        if (restStat.getCode() == YesNoEnum.YES.getValue()) {
            Integer pageNum = restStat.getData().getOrDefault("pageNum", 0);
            Integer total = restStat.getData().getOrDefault("total", 0);
            //获取事项全列表
            Rest<List<MatterEntity>> matterAllRest = this.getMatterAllListByGOV(params, pageNum, context);

            if (matterAllRest.getCode() == YesNoEnum.YES.getValue()) {
                return matterAllRest.getData();
            }
        }
        return new ArrayList<MatterEntity>();

    }


    @Override
    public Rest<String> syncMatterBySiteId(SiteEntity siteEntity, Context context) {
        //根据站点中区域编码 查询当前是否已经存在事项
        //根据站点区域编码查询政务网事项列表
        AreaEntity areaEntity = areaService.getCache(siteEntity.getAreaCode());
        //String dxType = AreaLevelDxTypeEnum.getByValue(areaEntity == null ? 2 : areaEntity.getAreaLevel()).getDesc();
        // String dxType="2";
        //String dxType = "3";
        List<MatterEntity> govMatterList = new ArrayList<>();
        List<DeptEntity> deptEntities = deptService.find(new DeptQuery().siteId(siteEntity.getId()).source(0));
        for (DeptEntity deptEntity : deptEntities) {
            HashMap<String, String> params = new HashMap<>();
            params.put("areaCode", areaEntity.getAreaCode());
            params.put("dxType", "6");
            params.put("deptCode", deptEntity.getDeptNumber());
            params.put("searchtext", "");
            params.put("taskType", "");
            List<MatterEntity> deptMatterList = this.getMatters(params, context);

            if (!ObjectUtils.isEmpty(deptMatterList)) {
                govMatterList.addAll(deptMatterList);
            }
        }
     /*   HashMap<String, String> params = new HashMap<>();
        params.put("dxType", dxType);
        params.put("areaCode", siteEntity.getAreaCode());
        params.put("deptCode", "");
        params.put("searchtext", "");
        params.put("taskType", "");
        params.put("pageno", "1");
        params.put("type", "2");

        List<MatterEntity> govMatterList = this.getMatters(params, context);
        dxType = "3";
        params = new HashMap<>();
        params.put("dxType", dxType);
        params.put("areaCode", siteEntity.getAreaCode());
        params.put("deptCode", "");
        params.put("searchtext", "");
        params.put("taskType", "");
        params.put("pageno", "1");
        params.put("type", "2");

        List<MatterEntity> mattersTwo = this.getMatters(params, context);
        govMatterList.addAll(mattersTwo);*/

        List<MatterEntity> localMatterList = matterService.find(new MatterQuery().areaCode(siteEntity.getAreaCode()));
        List<MatterEntity> subList = this.subList(govMatterList, localMatterList);
        log.info("抓取事项总数：{}需要添加事项数量：{}", govMatterList.size(), subList.size());
        subList = subList.stream().distinct().collect(Collectors.toList());
        log.info("需要添加事项过滤后数量：{}" , subList.size());
        //差集进行插入并更新详细数据
        if (!ObjectUtils.isEmpty(subList)) {
            for (MatterEntity matterEntity : subList) {
                DeptEntity deptCache = deptService.getExtCache(matterEntity.getDeptCode());
                matterEntity.setDeptName(deptCache == null ? "" : deptCache.getName());
                matterService.save(matterEntity, context);
            }

       /*     List<List<MatterEntity>> partition = ListUtil.partition(subList, 100);
            for (List<MatterEntity> matterEntityList : partition) {
                log.info("insert subList size:" + matterEntityList.size());
                int count = matterService.save(matterEntityList, context);
                log.info("insert subList size success:" + count);
            }*/
        }



       /* Rest<Map<String, Integer>> restStat = MatterHtmlParseUtil.statSiteMatterDeptCount(params, url);
        if (restStat.getCode() == YesNoEnum.YES.getValue()) {
            Integer pageNum = restStat.getData().getOrDefault("pageNum", 0);
            Integer total = restStat.getData().getOrDefault("total", 0);
            //获取事项全列表
            Rest<List<MatterEntity>> matterAllRest = this.getMatterAllListByGOV(params, pageNum, context);

            if (matterAllRest.getCode() == YesNoEnum.YES.getValue()) {
                List<MatterEntity> govMatterList = matterAllRest.getData();
                List<MatterEntity> localMatterList = matterService.find(new MatterQuery().areaCode(siteEntity.getAreaCode()));
                List<MatterEntity> subList = this.subList(govMatterList, localMatterList);
                log.info("需要添加事项数量====" + subList.size());
                //差集进行插入并更新详细数据
                if (!ObjectUtils.isEmpty(subList)) {
                    for (MatterEntity matterEntity : subList) {
                        DeptEntity deptCache = deptService.getExtCache(matterEntity.getDeptCode());
                        matterEntity.setDeptName(deptCache == null ? "" : deptCache.getName());
                        matterService.save(matterEntity, context);
                    }

                    List<List<MatterEntity>> partition = ListUtil.partition(subList, 100);
                    for (List<MatterEntity> matterEntityList : partition) {
                        log.info("insert subList size:" + matterEntityList.size());
                        int count = matterService.save(matterEntityList, context);
                        log.info("insert subList size success:" + count);
                    }
                }
            }
        }
*/
        return Rest.ok("同步事项条数成功！");
    }

    @Override
    public void deleteBysiteIdAndSource(Long siteId, Integer source, Context context) {
        Map<String, Object> condition = new HashMap<>();
        condition.put("siteId", siteId);
        condition.put("source", source);
        this.dao.delete(condition);
    }


    public List<MatterEntity> subList(List<MatterEntity> firstList, List<MatterEntity> secondList) {
        Set<String> secondSet = secondList.parallelStream().map(e -> e.getMatterNo()).collect(Collectors.toSet());
        return firstList.parallelStream().filter(item -> !secondSet.contains(item.getMatterNo())).collect(Collectors.toList());
    }


}