package com.mortals.xhx.module.workman.service.impl;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ZipUtil;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.service.impl.AbstractCRUDCacheServiceImpl;
import com.mortals.framework.util.SecurityUtil;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.common.utils.EncodeUtil;
import com.mortals.xhx.module.workman.dao.WorkmanDao;
import com.mortals.xhx.module.workman.model.WorkmanEntity;
import com.mortals.xhx.module.workman.model.WorkmanQuery;
import com.mortals.xhx.module.workman.service.WorkmanService;
import lombok.Getter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.Date;
import java.util.List;

/**
 * WorkmanService
 * 工作人员 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("workmanService")
public class WorkmanServiceImpl extends AbstractCRUDCacheServiceImpl<WorkmanDao, WorkmanEntity, Long> implements WorkmanService {

    @Autowired
    private UserService userService;
    @Autowired
    private UploadService uploadService;

    @Value("${upload.path}")
    @Getter
    private String filePath;
    @Autowired
    private WorkmanService workmanService;


    @Override
    protected void saveBefore(WorkmanEntity entity, Context context) throws AppException {
        if (StringUtils.isNull(entity.getLoginName())) {
            throw new AppException("账号不能为空！");
        }

        if (StringUtils.isNull(entity.getLoginPwd())) {
            throw new AppException("密码不能为空！");
        }
        int count = this.count(new WorkmanQuery().loginName(entity.getLoginName()), context);
        if (count > 0) {
            throw new AppException("账号已存在！");
        }

        doHandlerUser(entity);

        super.saveBefore(entity, context);
    }


    private void doHandlerUser(WorkmanEntity entity) throws AppException {
        if (StringUtils.isNotEmpty(entity.getLoginPwd())) {
            try {
                entity.setLoginPwd(SecurityUtil.md5DoubleEncoding(entity.getLoginPwd()));
            } catch (Exception e) {
                throw new AppException("密码转换异常");
            }
        } else {
            entity.setLoginPwd(null);
        }
        //判断党员是否其它选项，如果
    }

    @Override
    protected void updateBefore(WorkmanEntity entity, Context context) throws AppException {
        doHandlerUser(entity);
        super.updateBefore(entity, context);
    }

    @Override
    protected void saveAfter(WorkmanEntity entity, Context context) throws AppException {
        super.saveAfter(entity, context);

    }


    @Override
    public WorkmanEntity doLogin(String loginName, String password, String loginIp) throws AppException {
        WorkmanEntity workmanEntity = this.selectOne(new WorkmanQuery().loginName(loginName));
        if (workmanEntity == null || !workmanEntity.getLoginName().equals(loginName)) {
            throw new AppException("用户名不存在！");
        }
        try {
            if (!workmanEntity.getLoginPwd().equals(SecurityUtil.md5DoubleEncoding(password))) {
                throw new AppException("登录密码错误！");
            }
        } catch (Exception e) {
            throw new AppException("密码验认出错！", e);
        }
//        WorkmanEntity update = new WorkmanEntity();
//        update.setId(workmanEntity.getId());
//        this.dao.update(update);
        return workmanEntity;
    }

    @Override
    public boolean updateUserPwd(String loginName, String oldPwd, String newPwd) throws AppException {
        WorkmanEntity sysUser = this.selectOne(new WorkmanQuery().loginName(loginName));
        if (sysUser == null || !sysUser.getLoginName().equals(loginName)) {
            throw new AppException("帐号错误！");
        }
        try {
            if (!sysUser.getLoginPwd().equals(SecurityUtil.md5DoubleEncoding(oldPwd))) {
                throw new AppException("原始密码错误！");
            }
        } catch (Exception e) {
            throw new AppException("密码验认出错！", e);
        }
        try {
            sysUser.setLoginPwd(SecurityUtil.md5DoubleEncoding(newPwd));
        } catch (Exception e) {
            throw new AppException("密码转换异常！", e);
        }
        dao.update(sysUser);
        return true;
    }

    /**
     * @param loginName
     * @param newPwd
     * @return
     * @throws AppException
     */
    @Override
    public boolean updateUserPwd(String loginName, String newPwd) throws AppException {
        WorkmanEntity sysUser = this.selectOne(new WorkmanQuery().loginName(loginName));
        if (sysUser == null || !sysUser.getLoginName().equals(loginName)) {
            throw new AppException("帐号错误！");
        }
        try {
            sysUser.setLoginPwd(SecurityUtil.md5DoubleEncoding(newPwd));
        } catch (Exception e) {
            throw new AppException("密码转换异常！", e);
        }
        dao.update(sysUser);
        return true;
    }

    /**
     * @param picfilePath
     * @return
     * @throws AppException
     */
    @Override
    public Rest<Void> doImportPic(String picfilePath) throws AppException {
        String disPath = filePath + "/pics/" + FileUtil.getPrefix(picfilePath).trim();
        String targetFilePath = uploadService.getFilePath(picfilePath);
        if (FileUtil.isEmpty(new File(targetFilePath))) {
            throw new AppException("文件不存在！");
        }
        //解压图片
        String fileEncode = "UTF-8";
        try {
            fileEncode = EncodeUtil.getEncode(targetFilePath, true);
        } catch (Exception e) {
            log.error("异常", e);
        }
        ZipUtil.unzip(targetFilePath, disPath, Charset.forName(fileEncode));
        //读取目录下的所有图片
        File[] files = FileUtil.ls(disPath);
        for (File file : files) {
            if (file.isFile()) {
                updateWorkmanPic(file);
            }
            if (file.isDirectory()) {
                File[] subFiles = file.listFiles();
                for (File subFile : subFiles) {
                    updateWorkmanPic(subFile);
                }
            }
        }

        return Rest.ok("解析成功");
    }

    private void updateWorkmanPic(File file) {
        String userName = FileUtil.getPrefix(file).trim();
        //根据用户名查询工作人员名称
        WorkmanEntity workmanEntity = workmanService.selectOne(new WorkmanQuery().name(userName));
        if (!ObjectUtils.isEmpty(workmanEntity)) {
            String newName = "/file/fileupload/" + new Date().getTime() + "." + FileUtil.getSuffix(file);
            String filePath = uploadService.getFilePath(newName);
            try {
                boolean bool = com.mortals.framework.util.FileUtil.write(filePath, FileUtil.readBytes(file), true, true);
                if (bool) {
                    workmanEntity.setPhotoPath(newName);
                    workmanService.update(workmanEntity);
                }
            } catch (IOException e) {
                log.error("写入证照异常", e);
            }

        }
    }


    /**
     * @param list
     * @param updateSupport
     * @param context
     * @return
     */
    @Override
    public String importList(List<WorkmanEntity> list, Boolean updateSupport, Context context) {

        if (ObjectUtils.isEmpty(list)) {
            throw new AppException("导入数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();

        StringBuilder failureDetailMsg = new StringBuilder();
        for (WorkmanEntity entity : list) {
            try {
                entity.setCreateUserId(context == null ? 1L : context.getUser() == null ? 1L : context.getUser().getId());
                entity.setCreateTime(new Date());
                WorkmanEntity save = this.save(entity, context);
                if (save != null) {
                    successNum++;
                }
            } catch (Exception e) {
                log.error("导入异常", e);
                failureNum++;
                failureDetailMsg.append("\n");
                failureDetailMsg.append(String.format("导入工作人员数据失败！用户名:%s", entity.getName()));
                failureDetailMsg.append("\n");
            }
        }
        successMsg.insert(0, "数据导入成功！共 " + successNum + " 条");
        if (failureNum > 0) {
            failureMsg.insert(0, "导入失败！共 " + failureNum + " 条数据格式不正确");
            successMsg.append("\n");
            successMsg.append(failureMsg);
            successMsg.append(failureDetailMsg);
        } else {

        }
        return successMsg.toString();
    }

    public static void main(String[] args) throws Exception {
        System.out.println(SecurityUtil.md5DoubleEncoding(""));
    }


}